@@ -12,7 +12,6 @@ from pg8000.converters import (
     CHAR,
     CHAR_ARRAY,
     DATE,
-    DATETIME,
     FLOAT,
     FLOAT_ARRAY,
     INET,
@@ -34,7 +33,6 @@ from pg8000.converters import (
     TEXT,
     TEXT_ARRAY,
     TIME,
-    TIMEDELTA,
     TIMESTAMP,
     TIMESTAMPTZ,
     UNKNOWN,
@@ -881,63 +879,61 @@ class ArrayContentNotSupportedError(NotSupportedError):
 
 
 __all__ = [
-    Warning,
-    DataError,
-    DatabaseError,
-    connect,
-    InterfaceError,
-    ProgrammingError,
-    Error,
-    OperationalError,
-    IntegrityError,
-    InternalError,
-    NotSupportedError,
-    Connection,
-    Cursor,
-    Binary,
-    Date,
-    DateFromTicks,
-    Time,
-    TimeFromTicks,
-    Timestamp,
-    TimestampFromTicks,
-    BINARY,
-    PGInterval,
-    STRING,
-    DATETIME,
-    TIME,
-    BOOLEAN,
-    INTEGER,
-    BIGINT,
-    INTERVAL,
-    JSON,
-    JSONB,
-    NUMERIC,
-    NUMERIC_ARRAY,
-    NULLTYPE,
-    ROWID,
-    BOOLEAN_ARRAY,
-    BYTES,
-    CHAR,
-    CHAR_ARRAY,
-    DATE,
-    FLOAT,
-    FLOAT_ARRAY,
-    INET,
-    INT2VECTOR,
-    INTEGER_ARRAY,
-    MACADDR,
-    NAME,
-    NAME_ARRAY,
-    OID,
-    TEXT,
-    TEXT_ARRAY,
-    TIMEDELTA,
-    TIMESTAMP,
-    TIMESTAMPTZ,
-    UNKNOWN,
-    UUID_TYPE,
-    VARCHAR,
-    VARCHAR_ARRAY,
-    XID,
+    "BIGINT",
+    "BINARY",
+    "BOOLEAN",
+    "BOOLEAN_ARRAY",
+    "BYTES",
+    "Binary",
+    "CHAR",
+    "CHAR_ARRAY",
+    "Connection",
+    "Cursor",
+    "DATE",
+    "DataError",
+    "DatabaseError",
+    "Date",
+    "DateFromTicks",
+    "Error",
+    "FLOAT",
+    "FLOAT_ARRAY",
+    "INET",
+    "INT2VECTOR",
+    "INTEGER",
+    "INTEGER_ARRAY",
+    "INTERVAL",
+    "IntegrityError",
+    "InterfaceError",
+    "InternalError",
+    "JSON",
+    "JSONB",
+    "MACADDR",
+    "NAME",
+    "NAME_ARRAY",
+    "NULLTYPE",
+    "NUMERIC",
+    "NUMERIC_ARRAY",
+    "NotSupportedError",
+    "OID",
+    "OperationalError",
+    "PGInterval",
+    "ProgrammingError",
+    "ROWID",
+    "STRING",
+    "TEXT",
+    "TEXT_ARRAY",
+    "TIME",
+    "TIMESTAMP",
+    "TIMESTAMPTZ",
+    "Time",
+    "TimeFromTicks",
+    "Timestamp",
+    "TimestampFromTicks",
+    "UNKNOWN",
+    "UUID_TYPE",
+    "VARCHAR",
+    "VARCHAR_ARRAY",
+    "Warning",
+    "XID",
+    "connect",
 ]
